const { MongoClient, ObjectID } = require('mongodb');
const { EventEmitter } = require('events');

class DB extends EventEmitter {

  constructor(config, logger) {
    super();

    this.config = config;
    this.logger = logger;
    this.db = null;
    this.client = new MongoClient(this.config.url);
  }

  cleanup() {
    if (this.db) {
      this.db.close();
    }
  }

  async connect() {
    this.logger.info('MongoDB connection attempt');

    try {
      if (this.client && this.client.isConnected()) {
        this.logger.info('Already connected to MongoDB server');
        return;
      }

      this.client = await this.client.connect();
      this.logger.info('Connected successfully to MongoDB server');
      this.db = this.client.db(this.config.name);
      this.emit('connect');
    } catch (err) {
      this.logger.error(err.stack);
    }
  }

  async close(done) {
    if (this.client) {
      await this.client.close((err) => {
        this.client = null;
        this.db = null;
        if (done instanceof Function) {
          done(err);
        }
        this.logger.info('Connection to MongoDB server was closed');
      });
    }
  }

  get() {
    if (!this.db) {
      throw Error('MongoDB connection is dead');
    }
    return this.db;
  }

  objectId(str) {
    return new ObjectID(str);
  }

  async reconnectLooper() {
    if (!this.client.isConnected()) {
      await this.connect();
    }
    setInterval((async () => {
      if (!this.client.isConnected()) {
        await this.connect();
      }
    }), 5000);
  }
}

module.exports = DB;
